@@ -117,25 +117,31 @@ fn candidates_from_setup_py(
     net_access: bool,
 ) -> Result<Option<WatchCandidate>, Box<dyn std::error::Error>> {
     // Use Python to extract project name and version from setup.py
+    // We monkey-patch setup() to capture the arguments
     let script = r#"
 import sys
 import os
+import setuptools
+
+setup_args = {}
+
+def capture_setup(**kwargs):
+    setup_args.update(kwargs)
+
+# Patch setuptools.setup and the distutils compatibility layer
+setuptools.setup = capture_setup
+setuptools._distutils.core.setup = capture_setup
+
+# Execute the setup.py file
 sys.path.insert(0, os.path.dirname(sys.argv[1]))
-try:
-    from setuptools import setup
-except ImportError:
-    pass
-from distutils.core import run_setup
-try:
-    result = run_setup(sys.argv[1], stop_after='config')
-    name = result.get_name()
-    version = result.get_version()
-    if name:
-        print(name)
-        if version:
-            print(version)
-except:
-    pass
+with open(sys.argv[1], 'r') as f:
+    code = compile(f.read(), sys.argv[1], 'exec')
+    exec(code)
+
+if 'name' in setup_args:
+    print(setup_args['name'])
+    if 'version' in setup_args:
+        print(setup_args['version'])
 "#;
 
     let output = Command::new("python3")
