@@ -1,12 +1,14 @@
-use breezyshim::dirty_tracker::DirtyTracker;
-use breezyshim::tree::{CommitError, Error as TreeError, WorkingTree};
-use debian_analyzer::debmutateshim::{
-    format_relations, parse_relations, ControlEditor, ControlLikeEditor, Deb822Paragraph,
-};
+use breezyshim::dirty_tracker::DirtyTreeTracker;
+use breezyshim::error::Error;
+use breezyshim::tree::WorkingTree;
+use breezyshim::workingtree::GenericWorkingTree;
+use debian_analyzer::control::TemplatedControlEditor;
 use debian_analyzer::{
     add_changelog_entry, apply_or_revert, certainty_sufficient, get_committer, ApplyError,
     Certainty, ChangelogError,
 };
+use debian_control::control::Binary;
+use debian_control::fields::MultiArch;
 use debversion::Version;
 use lazy_regex::regex_captures;
 use lazy_static::lazy_static;
@@ -14,7 +16,6 @@ use reqwest::blocking::Client;
 use serde::Deserialize;
 use serde_yaml::from_value;
 use std::collections::HashMap;
-use std::error::Error;
 use std::fs;
 use std::io::Read;
 use std::io::Write;
@@ -26,26 +27,48 @@ const USER_AGENT: &str = concat!("apply-multiarch-hints/", env!("CARGO_PKG_VERSI
 
 const DEFAULT_VALUE_MULTIARCH_HINT: i32 = 100;
 
-lazy_static! {
-    static ref MULTIARCH_HINTS_VALUE: HashMap<&'static str, i32> = {
-        let mut map = HashMap::new();
-        map.insert("ma-foreign", 20);
-        map.insert("file-conflict", 50);
-        map.insert("ma-foreign-library", 20);
-        map.insert("dep-any", 20);
-        map.insert("ma-same", 20);
-        map.insert("arch-all", 20);
-        map.insert("ma-workaround", 20);
-        map
-    };
+#[derive(Debug, Clone, Copy, std::hash::Hash, PartialEq, Eq)]
+pub enum HintKind {
+    MaForeign,
+    FileConflict,
+    MaForeignLibrary,
+    DepAny,
+    MaSame,
+    ArchAll,
+    MaWorkaround,
 }
 
-pub fn calculate_value(hints: &[&str]) -> i32 {
-    hints
-        .iter()
-        .map(|hint| *MULTIARCH_HINTS_VALUE.get(hint).unwrap_or(&0))
-        .sum::<i32>()
-        + DEFAULT_VALUE_MULTIARCH_HINT
+impl std::str::FromStr for HintKind {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
+            "ma-foreign" => Ok(HintKind::MaForeign),
+            "file-conflict" => Ok(HintKind::FileConflict),
+            "ma-foreign-library" => Ok(HintKind::MaForeignLibrary),
+            "dep-any" => Ok(HintKind::DepAny),
+            "ma-same" => Ok(HintKind::MaSame),
+            "arch-all" => Ok(HintKind::ArchAll),
+            "ma-workaround" => Ok(HintKind::MaWorkaround),
+            _ => Err(format!("Invalid hint kind: {:?}", s)),
+        }
+    }
+}
+
+fn hint_value(hint: HintKind) -> i32 {
+    match hint {
+        HintKind::MaForeign => 20,
+        HintKind::FileConflict => 50,
+        HintKind::MaForeignLibrary => 20,
+        HintKind::DepAny => 20,
+        HintKind::MaSame => 20,
+        HintKind::ArchAll => 20,
+        HintKind::MaWorkaround => 20,
+    }
+}
+
+pub fn calculate_value(hints: &[HintKind]) -> i32 {
+    hints.iter().map(|hint| hint_value(*hint)).sum::<i32>() + DEFAULT_VALUE_MULTIARCH_HINT
 }
 
 fn format_system_time(system_time: SystemTime) -> String {
@@ -92,7 +115,7 @@ pub struct Hint {
 
 impl Hint {
     pub fn kind(&self) -> &str {
-        self.link.split('#').last().unwrap()
+        self.link.split('#').next_back().unwrap()
     }
 }
 
@@ -175,14 +198,17 @@ format: blah
     }
 }
 
-pub fn cache_download_multiarch_hints(url: Option<&str>) -> Result<Vec<u8>, Box<dyn Error>> {
+pub fn cache_download_multiarch_hints(
+    url: Option<&str>,
+) -> Result<Vec<u8>, Box<dyn std::error::Error>> {
     let cache_home = if let Ok(xdg_cache_home) = std::env::var("XDG_CACHE_HOME") {
         Path::new(&xdg_cache_home).to_path_buf()
     } else if let Ok(home) = std::env::var("HOME") {
         Path::new(&home).join(".cache")
     } else {
         log::warn!("Unable to find cache directory, not caching");
-        return download_multiarch_hints(url, None).map(|x| x.unwrap());
+        return download_multiarch_hints(url, None)?
+            .ok_or_else(|| "Expected download data but got None".into());
     };
     let cache_dir = cache_home.join("lintian-brush");
     fs::create_dir_all(&cache_dir)?;
@@ -209,7 +235,7 @@ pub fn cache_download_multiarch_hints(url: Option<&str>) -> Result<Vec<u8>, Box<
 pub fn download_multiarch_hints(
     url: Option<&str>,
     since: Option<SystemTime>,
-) -> Result<Option<Vec<u8>>, Box<dyn Error>> {
+) -> Result<Option<Vec<u8>>, Box<dyn std::error::Error>> {
     let url = url.unwrap_or(MULTIARCH_HINTS_URL);
     let client = Client::builder().user_agent(USER_AGENT).build()?;
     let mut request = client.get(url).header("Accept-Encoding", "identity");
@@ -253,79 +279,74 @@ impl OverallResult {
         let kinds = self
             .changes
             .iter()
-            .map(|x| x.hint.kind())
+            .map(|x| x.hint.kind().parse().unwrap())
             .collect::<Vec<_>>();
         calculate_value(&kinds)
     }
 }
 
-fn apply_hint_ma_foreign(binary: &mut Deb822Paragraph, _hint: &Hint) -> Option<String> {
-    if binary.get("Multi-Arch").as_deref() != Some("foreign") {
-        binary.set("Multi-Arch", "foreign");
+fn apply_hint_ma_foreign(binary: &mut Binary, _hint: &Hint) -> Option<String> {
+    if binary.multi_arch() != Some(MultiArch::Foreign) {
+        binary.set_multi_arch(Some(MultiArch::Foreign));
         Some("Add Multi-Arch: foreign.".to_string())
     } else {
         None
     }
 }
 
-fn apply_hint_ma_foreign_lib(binary: &mut Deb822Paragraph, _hint: &Hint) -> Option<String> {
-    if binary.get("Multi-Arch").as_deref() == Some("foreign") {
-        binary.remove("Multi-Arch");
+fn apply_hint_ma_foreign_lib(binary: &mut Binary, _hint: &Hint) -> Option<String> {
+    if binary.multi_arch() == Some(MultiArch::Foreign) {
+        binary.set_multi_arch(None);
         Some("Drop Multi-Arch: foreign.".to_string())
     } else {
         None
     }
 }
 
-fn apply_hint_file_conflict(binary: &mut Deb822Paragraph, _hint: &Hint) -> Option<String> {
-    if binary.get("Multi-Arch").as_deref() == Some("same") {
-        binary.remove("Multi-Arch");
+fn apply_hint_file_conflict(binary: &mut Binary, _hint: &Hint) -> Option<String> {
+    if binary.multi_arch() == Some(MultiArch::Same) {
+        binary.set_multi_arch(None);
         Some("Drop Multi-Arch: same.".to_string())
     } else {
         None
     }
 }
 
-fn apply_hint_ma_same(binary: &mut Deb822Paragraph, _hint: &Hint) -> Option<String> {
-    if binary.get("Multi-Arch").as_deref() == Some("same") {
+fn apply_hint_ma_same(binary: &mut Binary, _hint: &Hint) -> Option<String> {
+    if binary.multi_arch() == Some(MultiArch::Same) {
         return None;
     }
-    binary.set("Multi-Arch", "same");
+    binary.set_multi_arch(Some(MultiArch::Same));
     Some("Add Multi-Arch: same.".to_string())
 }
 
-fn apply_hint_arch_all(binary: &mut Deb822Paragraph, _hint: &Hint) -> Option<String> {
-    if binary.get("Architecture").as_deref() == Some("all") {
+fn apply_hint_arch_all(binary: &mut Binary, _hint: &Hint) -> Option<String> {
+    if binary.architecture().as_deref() == Some("all") {
         return None;
     }
-    binary.set("Architecture", "all");
+    binary.set_architecture(Some("all"));
     Some("Make package Architecture: all.".to_string())
 }
 
-fn apply_hint_dep_any(binary: &mut Deb822Paragraph, hint: &Hint) -> Option<String> {
+fn apply_hint_dep_any(binary: &mut Binary, hint: &Hint) -> Option<String> {
     if let Some((_whole, binary_package, dep)) = regex_captures!(
         r"(.*) could have its dependency on (.*) annotated with :any",
         hint.description.as_str()
     ) {
-        assert_eq!(binary_package, binary.get("Package").unwrap());
+        assert_eq!(binary_package, binary.name().unwrap());
 
         let mut changed = false;
-        if let Some(depends) = binary.get("Depends") {
-            let mut relations = parse_relations(depends.as_str());
-            for (_head_whitespace, relation, _tail_whitespace) in &mut relations {
-                for r in relation {
-                    if r.name == dep && r.archqual.as_deref() != Some("any") {
-                        r.archqual = Some("any".to_string());
+        if let Some(depends) = binary.depends() {
+            for entry in depends.entries() {
+                for mut r in entry.relations() {
+                    if r.name() == dep && r.archqual().as_deref() != Some("any") {
+                        r.set_archqual("any");
                         changed = true;
                     }
                 }
             }
             if changed {
-                let relations = relations
-                    .iter()
-                    .map(|(f, m, t)| (f.as_str(), m.as_slice(), t.as_str()))
-                    .collect::<Vec<_>>();
-                binary.set("Depends", format_relations(relations.as_slice()).as_str());
+                binary.set_depends(Some(&depends));
                 Some(format!("Add :any qualifier for {} dependency.", dep))
             } else {
                 None
@@ -339,14 +360,14 @@ fn apply_hint_dep_any(binary: &mut Deb822Paragraph, hint: &Hint) -> Option<Strin
     }
 }
 
-fn apply_hint_ma_workaround(binary: &mut Deb822Paragraph, hint: &Hint) -> Option<String> {
+fn apply_hint_ma_workaround(binary: &mut Binary, hint: &Hint) -> Option<String> {
     if let Some((_whole, binary_package)) = regex_captures!(
         r"(.*) should be Architecture: any \+ Multi-Arch: same",
         hint.description.as_str()
     ) {
-        assert_eq!(binary_package, binary.get("Package").unwrap());
-        binary.set("Multi-Arch", "same");
-        binary.set("Architecture", "any");
+        assert_eq!(binary_package, binary.name().unwrap());
+        binary.set_multi_arch(Some(MultiArch::Same));
+        binary.set_architecture(Some("any"));
         Some("Add Multi-Arch: same and set Architecture: any.".to_string())
     } else {
         log::warn!("Unable to parse ma-workaround hint: {:?}", hint.description);
@@ -357,7 +378,7 @@ fn apply_hint_ma_workaround(binary: &mut Deb822Paragraph, hint: &Hint) -> Option
 struct Applier {
     kind: &'static str,
     certainty: Certainty,
-    cb: fn(&mut Deb822Paragraph, &Hint) -> Option<String>,
+    cb: fn(&mut Binary, &Hint) -> Option<String>,
 }
 
 lazy_static! {
@@ -417,12 +438,31 @@ fn changes_by_description(changes: &[Change]) -> HashMap<String, Vec<String>> {
 
 #[derive(Debug)]
 pub enum OverallError {
-    TreeError(TreeError),
+    BrzError(Error),
     NotDebianPackage(std::path::PathBuf),
     Other(String),
-    Python(pyo3::PyErr),
     NoWhoami,
     NoChanges,
+    GeneratedFile(std::path::PathBuf),
+    FormattingUnpreservable(std::path::PathBuf),
+}
+
+impl From<debian_analyzer::editor::EditorError> for OverallError {
+    fn from(e: debian_analyzer::editor::EditorError) -> Self {
+        match e {
+            debian_analyzer::editor::EditorError::GeneratedFile(p, _) => {
+                OverallError::GeneratedFile(p)
+            }
+            debian_analyzer::editor::EditorError::FormattingUnpreservable(p, _) => {
+                OverallError::FormattingUnpreservable(p)
+            }
+            debian_analyzer::editor::EditorError::BrzError(e) => OverallError::BrzError(e),
+            debian_analyzer::editor::EditorError::IoError(e) => OverallError::Other(e.to_string()),
+            debian_analyzer::editor::EditorError::TemplateError(p, _e) => {
+                OverallError::GeneratedFile(p)
+            }
+        }
+    }
 }
 
 impl std::fmt::Display for OverallError {
@@ -431,8 +471,13 @@ impl std::fmt::Display for OverallError {
             OverallError::NotDebianPackage(p) => {
                 write!(f, "{} is not a Debian package.", p.display())
             }
-            OverallError::TreeError(e) => write!(f, "{}", e),
-            OverallError::Python(e) => write!(f, "{}", e),
+            OverallError::GeneratedFile(p) => {
+                write!(f, "Generated file: {}", p.display())
+            }
+            OverallError::FormattingUnpreservable(p) => {
+                write!(f, "Formatting unpreservable: {}", p.display())
+            }
+            OverallError::BrzError(e) => write!(f, "{}", e),
             OverallError::NoWhoami => write!(f, "No committer configured."),
             OverallError::NoChanges => write!(f, "No changes to apply."),
             OverallError::Other(e) => write!(f, "{}", e),
@@ -442,18 +487,13 @@ impl std::fmt::Display for OverallError {
 
 impl std::error::Error for OverallError {}
 
-impl From<TreeError> for OverallError {
-    fn from(e: TreeError) -> Self {
-        OverallError::TreeError(e)
-    }
-}
-
-impl From<CommitError> for OverallError {
-    fn from(e: CommitError) -> Self {
+impl From<Error> for OverallError {
+    fn from(e: Error) -> Self {
         match e {
-            CommitError::PointlessCommit => OverallError::NoChanges,
-            CommitError::NoWhoami => OverallError::NoWhoami,
-            CommitError::Other(e) => OverallError::Python(e),
+            Error::PointlessCommit => OverallError::NoChanges,
+            Error::NoWhoami => OverallError::NoWhoami,
+            Error::Other(e) => OverallError::Other(e.to_string()),
+            e => OverallError::BrzError(e),
         }
     }
 }
@@ -467,32 +507,50 @@ impl From<ChangelogError> for OverallError {
     }
 }
 
+/// Configuration options for applying multiarch hints
+#[derive(Debug, Clone)]
+pub struct ApplyMultiarchHintsConfig {
+    pub minimum_certainty: Option<Certainty>,
+    pub committer: Option<String>,
+    pub update_changelog: bool,
+    pub allow_reformatting: Option<bool>,
+}
+
+impl Default for ApplyMultiarchHintsConfig {
+    fn default() -> Self {
+        Self {
+            minimum_certainty: None,
+            committer: None,
+            update_changelog: true,
+            allow_reformatting: None,
+        }
+    }
+}
+
 pub fn apply_multiarch_hints(
-    local_tree: &WorkingTree,
+    local_tree: &GenericWorkingTree,
     subpath: &std::path::Path,
     hints: &HashMap<&str, Vec<&Hint>>,
-    minimum_certainty: Option<Certainty>,
-    committer: Option<String>,
-    dirty_tracker: Option<&DirtyTracker>,
-    update_changelog: bool,
-    allow_reformatting: Option<bool>,
+    dirty_tracker: Option<&mut DirtyTreeTracker>,
+    config: &ApplyMultiarchHintsConfig,
 ) -> Result<OverallResult, OverallError> {
-    let minimum_certainty = minimum_certainty.unwrap_or(Certainty::Certain);
-    let basis_tree = local_tree.basis_tree();
+    let minimum_certainty = config.minimum_certainty.unwrap_or(Certainty::Certain);
+    let basis_tree = local_tree.basis_tree().map_err(OverallError::BrzError)?;
     let (changes, _tree_changes, mut specific_files) = match apply_or_revert(
         local_tree,
         subpath,
         &basis_tree,
         dirty_tracker,
-        |path| -> Result<Vec<Change>, ()> {
+        |path| -> Result<Vec<Change>, OverallError> {
             let mut changes: Vec<Change> = vec![];
 
             let control_path = path.join("debian/control");
 
-            let editor = ControlEditor::open(Some(control_path.as_path()), allow_reformatting);
+            let editor = TemplatedControlEditor::open(control_path.as_path())
+                .map_err(|e| OverallError::Other(e.to_string()))?;
 
             for mut binary in editor.binaries() {
-                let package = binary.get("Package").unwrap();
+                let package = binary.name().unwrap();
                 if let Some(hints) = hints.get(package.as_str()) {
                     for hint in hints {
                         let kind = hint.kind();
@@ -508,7 +566,7 @@ pub fn apply_multiarch_hints(
                         }
                         if let Some(description) = (applier.cb)(&mut binary, hint) {
                             changes.push(Change {
-                                binary: binary.get("Package").unwrap(),
+                                binary: binary.name().unwrap(),
                                 hint: (*hint).clone(),
                                 description,
                                 certainty: applier.certainty,
@@ -518,13 +576,13 @@ pub fn apply_multiarch_hints(
                 }
             }
 
-            std::mem::drop(editor);
+            editor.commit()?;
             Ok(changes)
         },
     ) {
         Ok(r) => r,
         Err(ApplyError::NoChanges(_)) => return Err(OverallError::NoChanges),
-        Err(ApplyError::TreeError(e)) => return Err(OverallError::TreeError(e)),
+        Err(ApplyError::BrzError(e)) => return Err(OverallError::BrzError(e)),
         Err(ApplyError::CallbackError(_)) => panic!("Unexpected callback error"),
     };
 
@@ -537,7 +595,7 @@ pub fn apply_multiarch_hints(
 
     let changelog_path = subpath.join("debian/changelog");
 
-    if update_changelog {
+    if config.update_changelog {
         add_changelog_entry(
             local_tree,
             changelog_path.as_path(),
@@ -555,18 +613,26 @@ pub fn apply_multiarch_hints(
     overall_description.push("\n".to_string());
     overall_description.push("Changes-By: apply-multiarch-hints\n".to_string());
 
-    let committer = committer.unwrap_or_else(|| get_committer(local_tree));
+    let committer = config
+        .committer
+        .clone()
+        .unwrap_or_else(|| get_committer(local_tree));
 
     let specific_files_ref = specific_files
         .as_ref()
         .map(|x| x.iter().map(|x| x.as_path()).collect::<Vec<_>>());
 
-    local_tree.commit(
-        overall_description.concat().as_str(),
-        Some(false),
-        Some(&committer),
-        specific_files_ref.as_deref(),
-    )?;
+    let mut builder = local_tree
+        .build_commit()
+        .message(overall_description.concat().as_str())
+        .allow_pointless(false)
+        .committer(&committer);
+
+    if let Some(specific_files) = specific_files_ref.as_deref() {
+        builder = builder.specific_files(specific_files);
+    }
+
+    builder.commit()?;
 
     Ok(OverallResult { changes })
 }
