@@ -1,8 +1,8 @@
 use crate::action::Action;
 use breezyshim::commit::NullCommitReporter;
 use breezyshim::error::Error as BrzError;
-use breezyshim::workingtree::WorkingTree;
-use deb822_lossless::lossless::Paragraph;
+use breezyshim::workingtree::{GenericWorkingTree, WorkingTree};
+use deb822_lossless::Paragraph;
 use debian_analyzer::editor::{Editor, EditorError, MutableTreeEdit};
 use debian_control::lossless::relations::{Entry, Relation, Relations};
 use debian_control::relations::VersionConstraint;
@@ -17,6 +17,15 @@ pub mod dummy_transitional;
 pub mod package_checker;
 use package_checker::{PackageChecker, UddPackageChecker};
 
+/// Represents a field change: (field_name, actions, description)
+pub type FieldChange = (String, Vec<Action>, String);
+
+/// Represents changes to a control paragraph: (paragraph_name, field_changes)
+pub type ParagraphChanges = (Option<String>, Vec<FieldChange>);
+
+/// A collection of control paragraph changes
+pub type ControlChanges = Vec<ParagraphChanges>;
+
 pub const DEFAULT_VALUE_MULTIARCH_HINT: usize = 30;
 
 pub fn note_changelog_policy(policy: bool, msg: &str) {
@@ -76,15 +85,15 @@ async fn drop_obsolete_depends(
     let mut to_remove = vec![];
     let mut to_replace = vec![];
     for (i, mut pkgrel) in entry.relations().enumerate() {
-        if let Some(replacement) = checker.replacement(&pkgrel.name()).await.unwrap() {
+        if let Some(replacement) = checker.replacement(&pkgrel.name()).await? {
             let parsed_replacement: Relations = replacement.parse().unwrap();
             if parsed_replacement.entries().count() > 1 {
                 log::warn!("Unable to replace multi-package {:?}", replacement);
             } else {
-                // If the replacement is already included in the entry, we can drop the old
-                // package.
                 let newrel: Entry = replacement.parse().unwrap();
                 if debian_analyzer::relations::is_relation_implied(&newrel, entry) {
+                    // If the replacement is already included in the entry, we can drop the old
+                    // package.
                     to_remove.push(i);
                     actions.push(Action::DropTransition(pkgrel));
                 } else {
@@ -283,7 +292,7 @@ fn drop_old_relations(
     compat_release: &str,
     upgrade_release: &str,
     keep_minimum_depends_versions: bool,
-) -> Vec<(Option<String>, Vec<(String, Vec<Action>, String)>)> {
+) -> ControlChanges {
     let mut actions = vec![];
     let mut source_actions = vec![];
 
@@ -316,20 +325,23 @@ fn drop_old_relations(
 }
 
 fn update_maintscripts(
-    wt: &WorkingTree,
+    wt: &GenericWorkingTree,
     debian_path: &Path,
     checker: &dyn PackageChecker,
     allow_reformatting: bool,
 ) -> Result<Vec<(PathBuf, Vec<MaintscriptAction>)>, ScrubObsoleteError> {
     let mut ret = vec![];
-    for entry in std::fs::read_dir(wt.abspath(debian_path).unwrap()).unwrap() {
-        let entry = entry.unwrap();
-        if !(entry.file_name() == "maintscript"
-            || entry
-                .file_name()
+    let debian_abs_path = wt.abspath(debian_path)?;
+    let dir_entries = std::fs::read_dir(debian_abs_path)?;
+
+    for entry in dir_entries {
+        let entry = entry?;
+        let file_name_str = entry.file_name();
+        if !(file_name_str == "maintscript"
+            || file_name_str
                 .to_str()
-                .unwrap()
-                .ends_with(".maintscript"))
+                .map(|s| s.ends_with(".maintscript"))
+                .unwrap_or(false))
         {
             continue;
         }
@@ -408,7 +420,7 @@ impl<'a> serde::Deserialize<'a> for MaintscriptAction {
 #[derive(serde::Serialize, serde::Deserialize)]
 pub struct ScrubObsoleteResult {
     specific_files: Vec<PathBuf>,
-    control_actions: Vec<(Option<String>, Vec<(String, Vec<Action>, String)>)>,
+    control_actions: ControlChanges,
     maintscript_removed: Vec<(PathBuf, Vec<MaintscriptAction>, String)>,
 }
 
@@ -469,7 +481,7 @@ impl ScrubObsoleteResult {
 }
 
 async fn _scrub_obsolete(
-    wt: &WorkingTree,
+    wt: &GenericWorkingTree,
     debian_path: &Path,
     compat_release: &str,
     upgrade_release: &str,
@@ -530,6 +542,7 @@ pub enum ScrubObsoleteError {
     EditorError(EditorError),
     BrzError(BrzError),
     SqlxError(sqlx::Error),
+    IoError(std::io::Error),
 }
 
 impl std::fmt::Display for ScrubObsoleteError {
@@ -541,6 +554,7 @@ impl std::fmt::Display for ScrubObsoleteError {
             ScrubObsoleteError::EditorError(e) => write!(f, "Editor error: {}", e),
             ScrubObsoleteError::BrzError(e) => write!(f, "Breezy error: {}", e),
             ScrubObsoleteError::SqlxError(e) => write!(f, "SQLx error: {}", e),
+            ScrubObsoleteError::IoError(e) => write!(f, "I/O error: {}", e),
         }
     }
 }
@@ -565,9 +579,15 @@ impl From<sqlx::Error> for ScrubObsoleteError {
     }
 }
 
+impl From<std::io::Error> for ScrubObsoleteError {
+    fn from(e: std::io::Error) -> Self {
+        ScrubObsoleteError::IoError(e)
+    }
+}
+
 /// Scrub obsolete entries.
 pub fn scrub_obsolete(
-    wt: WorkingTree,
+    wt: &GenericWorkingTree,
     subpath: &Path,
     compat_release: &str,
     upgrade_release: &str,
@@ -576,7 +596,7 @@ pub fn scrub_obsolete(
     keep_minimum_depends_versions: bool,
     #[allow(unused_variables)] transitions: Option<HashMap<String, String>>,
 ) -> Result<ScrubObsoleteResult, ScrubObsoleteError> {
-    let debian_path = if debian_analyzer::control_files_in_root(&wt, subpath) {
+    let debian_path = if debian_analyzer::control_files_in_root(wt, subpath) {
         subpath.to_path_buf()
     } else {
         subpath.join("debian")
@@ -585,7 +605,7 @@ pub fn scrub_obsolete(
     let rt = tokio::runtime::Runtime::new().unwrap();
 
     let result = rt.block_on(_scrub_obsolete(
-        &wt,
+        wt,
         &debian_path,
         compat_release,
         upgrade_release,
@@ -605,7 +625,7 @@ pub fn scrub_obsolete(
     let update_changelog = if let Some(update_changelog) = update_changelog {
         update_changelog
     } else if let Some(dch_guess) =
-        debian_analyzer::detect_gbp_dch::guess_update_changelog(&wt, &debian_path, None)
+        debian_analyzer::detect_gbp_dch::guess_update_changelog(wt, &debian_path, None)
     {
         note_changelog_policy(dch_guess.update_changelog, &dch_guess.explanation);
         dch_guess.update_changelog
@@ -627,7 +647,7 @@ pub fn scrub_obsolete(
             lines.extend(entries.iter().map(|x| format!("* {}", x)));
         }
         debian_analyzer::add_changelog_entry(
-            &wt,
+            wt,
             &changelog_path,
             lines
                 .iter()
@@ -651,7 +671,7 @@ pub fn scrub_obsolete(
     }
     lines.extend(["".to_string(), "Changes-By: deb-scrub-obsolete".to_string()]);
 
-    let committer = debian_analyzer::get_committer(&wt);
+    let committer = debian_analyzer::get_committer(wt);
 
     match wt
         .build_commit()
@@ -713,7 +733,7 @@ fn drop_obsolete_maintscript_entries(
 mod tests {
     use super::*;
     use async_trait::async_trait;
-    use deb822_lossless::lossless::Paragraph;
+    use deb822_lossless::Paragraph;
     use std::collections::{HashMap, HashSet};
 
     #[cfg(test)]
@@ -814,7 +834,7 @@ mod tests {
     }
 
     #[async_trait]
-    impl<'a> crate::package_checker::PackageChecker for DummyChecker<'a> {
+    impl crate::package_checker::PackageChecker for DummyChecker<'_> {
         fn release(&self) -> &str {
             "release"
         }
